/*********************************************************************************
 DataModule.h

	Interface for the DataModule class.
 
	Copyright @ 1997 by Glenn W. Bach. All rights reserved.

 ********************************************************************************/
 
#ifndef _H_DataModule
#define _H_DataModule

#include <JBroadcaster.h>
#include <JArray.h>

#include <ace/LSOCK_Stream.h>
#include <ace/UNIX_Addr.h>
#include <JMessageProtocol.h>	// template which requires ace includes

class GXRaggedFloatTable;
class GRaggedFloatTableData;
class JProcess;
class JInPipeStream;
class JOutPipeStream;
//class JIPCIOStream;
class JProgressDisplay;
class JString;
class istream;

class DataModule : virtual public JBroadcaster
{
public:

	static JBoolean Create(	DataModule** module,
							GXRaggedFloatTable* table, 
							GRaggedFloatTableData* data,
							const JCharacter* sysCmd);
	virtual ~DataModule();

protected:

	DataModule(	GXRaggedFloatTable* table, GRaggedFloatTableData* data,
				JProcess* 		process,
				JInPipeStream* input, JOutPipeStream* output,
				const int fd);

	virtual void	Receive(JBroadcaster* sender,
						const JBroadcaster::Message& message);

private:

	typedef JMessageProtocol<ACE_LSOCK_STREAM>	ProcessLink;

private:

	GXRaggedFloatTable* 		itsTable;
	GRaggedFloatTableData*		itsData;
	JInPipeStream*				itsInput;
	JOutPipeStream*				itsOutput;
	JBoolean					itsHeaderRead;
	JBoolean					itsSentData;
	JBoolean					itsDataIsDump;
	JSize						itsColNum;
	JIndex						itsColStart;
	JProgressDisplay*			itsPG;
	JIndex						itsCurrentRow;
	JProcess*					itsProcess;
//	JIPCLine*					itsProcessInput;
	JArray<JIndex>*				itsCols;
	ProcessLink*				itsLink;

private:

	void		HandleInput(const JString& str);
	void		HandleDataRead(const JString& str);
	void		HandlePrepareCols(const JSize cols);
	
	// not allowed

	DataModule(const DataModule& source);
	const DataModule& operator=(const DataModule& source);


};

#endif