/*********************************************************************************
 FitModule.h

	Interface for the FitModule class.
 
	Copyright @ 1997 by Glenn W. Bach. All rights reserved.

 ********************************************************************************/
 
#ifndef _H_FitModule
#define _H_FitModule

#include <JBroadcaster.h>
#include <JPtrArray.h>
#include <JArray.h>

#include <ace/LSOCK_Stream.h>
#include <ace/UNIX_Addr.h>
#include <JMessageProtocol.h>	// template which requires ace includes

class PlotDir;
class JProgressDisplay;
class JProcess;
class JString;
class JPlotDataBase;
class JInPipeStream;
class JOutPipeStream;

class FitModule : virtual public JBroadcaster
{
public:

	static JBoolean Create(	FitModule** module,
							PlotDir* dir,
							JPlotDataBase* fitData,
							const JCharacter* sysCmd);
	virtual ~FitModule();

protected:

	FitModule(	PlotDir* dir, JPlotDataBase* fitData, JProcess* process, 
				JInPipeStream* input, JOutPipeStream* output,
				const int fd);

	virtual void	Receive(JBroadcaster* sender,
						const JBroadcaster::Message& message);

private:

	typedef JMessageProtocol<ACE_LSOCK_STREAM>	ProcessLink;

private:

	PlotDir* 					itsDir;			// We don't own this.
	JPlotDataBase*				itsData;		// We don't own this.
	JBoolean					itsStatusRead;
	JBoolean					itsHeaderRead;
	JBoolean					itsFunctionRead;
	JBoolean					itsHasErrors;
	JBoolean					itsHasGOF;
	JSize						itsParmsCount;
	JIndex						itsCurrentRow;
	JProgressDisplay*			itsPG;
	JProcess*					itsProcess;
	JArray<JFloat>*				itsValues;
	JPtrArray<JString>*			itsNames;
 	JString*					itsFunction;
	JInPipeStream*				itsInput;
	ProcessLink*				itsLink;

private:

	void		HandleInput(JString& str);
	void		HandleDataRead(JString& str);
	void		HandleFit();
	
	// not allowed

	FitModule(const FitModule& source);
	const FitModule& operator=(const FitModule& source);


};

#endif