/******************************************************************************
 GLPrefsMgr.cc

	<Description>

	BASE CLASS = public JUNIXPrefsManager

	Copyright  1998 by Glenn W. Bach.  All rights reserved.
	
	Base code generated by Codemill v0.1.0

 *****************************************************************************/

#include <GLPrefsMgr.h>
#include <GLGlobals.h>

#include <JXWindow.h>
#include <JXPTPrinter.h>

#include <JStaticBuffer.h>
#include <JPtrArray-JString.h>
#include <JOrderedSet.h>

#include <jTypes.h>
#include <jStrStreamUtil.h>
#include <jDirUtil.h>

#include <unistd.h>
#include <strstream.h>
#include <iostream.h>
#include <jAssert.h>

const JFileVersion kCurrentPrefsFileVersion 	= 1;
const JCharacter* kGloveAppSig 					= "gGlove";

extern const JCharacter* kGloveVersionNumberStr;

/******************************************************************************
 Constructor

 *****************************************************************************/

GLPrefsMgr::GLPrefsMgr
	(
	JBoolean* isNew
	)
	:
   JUNIXPrefsManager(kGloveAppSig, kCurrentPrefsFileVersion, kTrue)
{
	*isNew = JPrefsManager::UpgradeData();
}

/******************************************************************************
 Destructor

 *****************************************************************************/

GLPrefsMgr::~GLPrefsMgr()
{
	SetData(kProgramVersionID, kGloveVersionNumberStr);
	SaveToDisk();
}

/******************************************************************************
 GetGloveVersionStr

 ******************************************************************************/

JString
GLPrefsMgr::GetGloveVersionStr()
	const
{
	JStaticBuffer data;
	if (GetData(kProgramVersionID, &data))
		{
		return JString(data);
		}
	else
		{
		return JString("< 0.8.0");		// didn't exist before this version
		}
}

/******************************************************************************
 UpgradeData (virtual protected)

 ******************************************************************************/

void
GLPrefsMgr::UpgradeData
	(
	const JBoolean		isNew,
	const JFileVersion	currentVersion
	)
{
	if (isNew)
		{
		}
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
GLPrefsMgr::Receive
	(
	JBroadcaster* 	sender, 
	const Message&	message
	)
{
//	if (sender == itsEditPrefsDialog && message.Is(JXDialogDirector::kDeactivated))
//		{
//		const JXDialogDirector::Deactivated* info =
//			dynamic_cast(const JXDialogDirector::Deactivated*, &message);
//		assert( info != NULL );
//		if (info->Successful())
//			{
//			UpdatePrefs(itsEditPrefsDialog);
//			}
//		itsEditPrefsDialog = NULL;
//		}
}

/******************************************************************************
 EditPrefs


 ******************************************************************************/

void
GLPrefsMgr::EditPrefs()
{
}

/******************************************************************************
 GetWindowSize (private)

 ******************************************************************************/

void
GLPrefsMgr::GetWindowSize
	(
	const JPrefID	id,
	JXWindow*		window
	)
	const
{
	if (IDValid(id))
		{
		JStaticBuffer data;
		GetData(id, &data);
		jistrstream(dataStream, data.GetData(), strlen(data));
		window->ReadGeometry(dataStream);
		}
}

/******************************************************************************
 SaveWindowSize (private)

 ******************************************************************************/

void
GLPrefsMgr::SaveWindowSize
	(
	const JPrefID	id,
	JXWindow*		window
	)
{
	ostrstream data;
	window->WriteGeometry(data);
	JPoint dtl = window->GetDesktopLocation();
	window->Place(dtl.x, dtl.y);

	SetData(id, data);
}

/******************************************************************************
 PrinterSetup

 ******************************************************************************/

void
GLPrefsMgr::WritePrinterSetup
	(
	JXPTPrinter* printer
	)
{
	ostrstream data;
	printer->WriteXPTSetup(data);
	SetData(kPTPrinterSetupID, data);
}

void
GLPrefsMgr::ReadPrinterSetup
	(
	JXPTPrinter* printer
	)
{
	if (IDValid(kPTPrinterSetupID))
		{
		JStaticBuffer data;
		GetData(kPTPrinterSetupID, &data);
		jistrstream(dataStream, data.GetData(), strlen(data));
		printer->ReadXPTSetup(dataStream);
		}
}
