/******************************************************************************
 GRaggedFloatTableData.h

	Interface for GRaggedFloatTableData class

	Copyright  1997 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GRaggedFloatTableData
#define _H_GRaggedFloatTableData

#include "JTableData.h"
#include <JPtrArray.h>
#include <JArray.h>
#include <jTypes.h>

class GRaggedFloatTableData : public JTableData
{
public:

	GRaggedFloatTableData(const JFloat& defValue);
	GRaggedFloatTableData(const GRaggedFloatTableData& source);

	virtual ~GRaggedFloatTableData();

	JBoolean CellValid(const JIndex row, const JIndex col) const;
	JBoolean CellValid(const JPoint& cell) const;
	JBoolean GetElement(const JIndex row, const JIndex col, JFloat* value) const;
	JBoolean GetElement(const JPoint& cell, JFloat* value) const;
	void	SetElement(const JIndex row, const JIndex col, const JFloat& data);
	void	SetElement(const JPoint& cell, const JFloat& data);

	void	GetRow(const JIndex index, JArray<JFloat>* rowData) const;
	void	SetRow(const JIndex index, const JArray<JFloat>& rowData);

	void	GetCol(const JIndex index, JOrderedSet<JFloat>* colData) const;
	void	SetCol(const JIndex index, const JOrderedSet<JFloat>& colData);
	JArray<JFloat>* GetColPointer(const JIndex index) const;

	void	InsertRow(const JIndex index, const JOrderedSet<JFloat>* initData = NULL);
	void	PrependRow();
	void	DuplicateRow(const JIndex index);
	void	RemoveRow(const JIndex index);
	void	RemoveAllRows();
	void	MoveRow(const JIndex origIndex, const JIndex newIndex);

	void	InsertCol(const JIndex index, const JOrderedSet<JFloat>* initData = NULL);
	void	PrependCol(const JOrderedSet<JFloat>* initData = NULL);
	void	AppendCol(const JOrderedSet<JFloat>* initData = NULL);
	void	DuplicateCol(const JIndex origIndex, const JIndex newIndex);
	void	RemoveCol(const JIndex index);
	void	RemoveAllCols();
	void	MoveCol(const JIndex origIndex, const JIndex newIndex);

	void	InsertElement(const JIndex row, const JIndex col, const JFloat value = 0);
	void	InsertElement(const JPoint& cell, const JFloat value = 0);
	void	PrependElement(const JIndex col, const JFloat value = 0);
	void	AppendElement(const JIndex col, const JFloat value = 0);
	void	DuplicateElement(const JIndex row, const JIndex col);
	void	DuplicateElement(const JPoint& cell);
	void	RemoveElement(const JIndex row, const JIndex col);
	void	RemoveElement(const JPoint& cell);
	void	RemoveAllElements(const JIndex col);
	void	MoveElement(const JIndex origRow, const JIndex origCol, 
						const JIndex newRow, const JIndex newCol);
	void	MoveElement(const JPoint& origCell, const JPoint& newCell);

	JSize	GetDataColCount() const;
	JBoolean ColumnValid(const JIndex index) const;
	JSize	GetDataRowCount(const JIndex index) const;
	JBoolean FindColumn(const JArray<JFloat>* array, JIndex *index) const;
	
	void	TurnOnBroadcast(const JBoolean on);

private:

	JPtrArray< JArray<JFloat> >*	itsCols;
	const JFloat					itsDefValue;
	JBoolean						itsBroadcast;

private:

	void	CheckBroadcast(const Message& message);

	void 	CreateCellIfNeeded(const JIndex row, const JIndex col);
	void 	CreateCellIfNeeded(const JPoint cell);
	JSize 	GetMaxRowCount();

	// not allowed

	const GRaggedFloatTableData&	operator=(const GRaggedFloatTableData& source);
	
public:

	static const JCharacter* kElementInserted;
	static const JCharacter* kElementRemoved;
	static const JCharacter* kDataChanged;
	
	class ElementInserted : public JBroadcaster::Message
		{
		public:
			
			ElementInserted(const JIndex row, const JIndex col)
				:
				JBroadcaster::Message(kElementInserted),
				itsRow(row),
				itsCol(col)
				{ };
				
			JIndex 
			GetRow() const
			{
				return itsRow;
			};
			
			JIndex
			GetCol() const
			{
				return itsCol;
			};
			
		private:
		
			JIndex itsRow;
			JIndex itsCol;
		};
		
	class ElementRemoved : public JBroadcaster::Message
		{
		public:
			
			ElementRemoved(const JIndex row, const JIndex col)
				:
				JBroadcaster::Message(kElementRemoved),
				itsRow(row),
				itsCol(col)
				{ };
				
			JIndex 
			GetRow() const
			{
				return itsRow;
			};
			
			JIndex
			GetCol() const
			{
				return itsCol;
			};
			
		private:
		
			JIndex itsRow;
			JIndex itsCol;
		};
	
	class DataChanged : public JBroadcaster::Message
		{
		public:
			
			DataChanged()
				:
				JBroadcaster::Message(kDataChanged)
				{ };
		};

};

#endif
