/******************************************************************************
 GRaggedStringTableData.h

	Interface for the GRaggedStringTableData Class

	Copyright  1997 Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GRaggedStringTableData
#define _H_GRaggedStringTableData

#include "JTableData.h"
#include <JString.h>
#include <JPtrArray.h>
#include <jTypes.h>

class GRaggedFloatTableData;

class GRaggedStringTableData : public JTableData
{
public:

	GRaggedStringTableData(const GRaggedFloatTableData* floatData, 
							const int precision);
	GRaggedStringTableData(const GRaggedStringTableData& source);

	virtual ~GRaggedStringTableData();

	JBoolean CellValid(const JIndex row, const JIndex col) const;
	JBoolean CellValid(const JPoint& cell) const;
	JBoolean GetElement(const JIndex row, const JIndex col, JString* str) const;
	JBoolean GetElement(const JPoint& cell, JString* str) const;
	void	SetElement(const JIndex row, const JIndex col, const JString& data);
	void	SetElement(const JPoint& cell, const JString& data);

	void	GetCol(const JIndex index, JPtrArray<JString>* colData) const;
	void	SetCol(const JIndex index, const JPtrArray<JString>& colData);

	void	InsertRow(const JIndex index);
	void	PrependRow();
	void	DuplicateRow(const JIndex index);
	void	RemoveRow(const JIndex index);
	void	RemoveAllRows();
	void	MoveRow(const JIndex origIndex, const JIndex newIndex);

	void	InsertCol(const JIndex index, const JPtrArray<JString>* initData = NULL);
	void	PrependCol(const JPtrArray<JString>* initData = NULL);
	void	AppendCol(const JPtrArray<JString>* initData = NULL);
	void	DuplicateCol(const JIndex origIndex, const JIndex newIndex);
	void	RemoveCol(const JIndex index);
	void	RemoveAllCols();
	void	MoveCol(const JIndex origIndex, const JIndex newIndex);

protected:

	virtual void	Receive(JBroadcaster* sender, const Message& message);
	
private:

	JPtrArray< JPtrArray<JString> >*	itsCols;
	const GRaggedFloatTableData*		itsFloatData;
	int							itsPrecision;

private:

	void	UpdateCell(const JPoint& cell);
	void	UpdateRow(const JIndex rowIndex);
	void	UpdateCol(const JIndex colIndex);
	void	InsertElement(const JIndex row, const JIndex col);
	void	RemoveElement(const JIndex row, const JIndex col);

	// not allowed

	const GRaggedStringTableData& operator=(const GRaggedStringTableData& source);
};

#endif
