/******************************************************************************
 GXChooseFileImportDialog.cc

	BASE CLASS = JXDialogDirector

	Copyright @ 1997 by Glenn W. Bach.

 ******************************************************************************/

#include "GXChooseFileImportDialog.h"
#include "GXPlotApp.h"
#include "TestFloatTableDirector.h"
#include "GLPrefsMgr.h"
#include "GLGlobals.h"

#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXTextMenu.h>
#include <JXStaticText.h>
#include <JXScrollbarSet.h>
#include <JXColormap.h>

#include <JPtrArray.h>
#include <JString.h>

#include <jStreamUtil.h>
#include <jXGlobals.h>

#include <fstream.h>
#include <jAssert.h>

extern GXPlotApp* gPlotApp;

const JSize  kFileByteCount				= 1000;
const JIndex kFileModulePrefsVersionID 	= 1;

/******************************************************************************
 Constructor

 ******************************************************************************/

GXChooseFileImportDialog::GXChooseFileImportDialog
	(
	TestFloatTableDirector* supervisor,
	const JString& 			filename
	)
	:
	JXDialogDirector(supervisor, kTrue),
	JPrefObject(GetPrefsMgr(), kFileModulePrefsID),
	itsDir(supervisor)
{
	BuildWindow(filename);
	JPrefObject::ReadPrefs();
}

/******************************************************************************
 Destructor

 ******************************************************************************/

GXChooseFileImportDialog::~GXChooseFileImportDialog()
{
	JPrefObject::WritePrefs();
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
GXChooseFileImportDialog::BuildWindow
	(
	const JString& filename
	)
{
	
// begin JXLayout

    JXWindow* window = new JXWindow(this, 330,230, "");
    assert( window != NULL );
    SetWindow(window);

    JXStaticText* obj1 =
        new JXStaticText("This is not a Glove data file.\nPlease select the appropriate filter.", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 10,10, 250,40);
    assert( obj1 != NULL );

    itsFilterMenu =
        new JXTextMenu("Filter:", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 50,160, 70,30);
    assert( itsFilterMenu != NULL );

    JXTextButton* okButton =
        new JXTextButton("OK", window,
                    JXWidget::kFixedRight, JXWidget::kFixedBottom, 210,200, 70,20);
    assert( okButton != NULL );
    okButton->SetShortcuts("^M");

    JXTextButton* cancelButton =
        new JXTextButton("Cancel", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 50,200, 70,20);
    assert( cancelButton != NULL );
    cancelButton->SetShortcuts("^[");

    itsReloadButton =
        new JXTextButton("Reload", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 130,200, 70,20);
    assert( itsReloadButton != NULL );

    JXScrollbarSet* obj2 =
        new JXScrollbarSet( window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 10,50, 310,100);
    assert( obj2 != NULL );

// end JXLayout

	window->SetTitle("Choose Import Filter");
	window->PlaceAsDialogWindow();
	UseModalPlacement(kFalse);
	SetButtons(okButton, cancelButton);

	const JSize dirModCount = itsDir->GetInternalModuleCount();
	for (JSize i = 1; i <= dirModCount; i++)
		{
		itsFilterMenu->AppendItem(itsDir->GetInternalModuleName(i));
		}

	JPtrArray<JString>* names = gPlotApp->GetImportModules();
	
	const JSize strCount = names->GetElementCount();
	
	for (JSize i = 1; i <= strCount; i++)
		{
		itsFilterMenu->AppendItem(*(names->NthElement(i)));
		}

	itsFilterIndex = 1;
	
	itsFilterMenu->SetToPopupChoice(kTrue, itsFilterIndex);
	itsFilterMenu->SetUpdateAction(JXMenu::kDisableNone);	
	ListenTo(itsFilterMenu);
	ListenTo(itsReloadButton);

	ifstream is(filename);
	JString text;
	text.Read(is, kFileByteCount);

	itsFileText = 
		new JXStaticText(text, kFalse, kFalse, 
			obj2, obj2->GetScrollEnclosure(), 
			JXWidget::kHElastic, JXWidget::kVElastic, 10,60, 310,90);
	assert(itsFileText != NULL);
	itsFileText->FitToEnclosure();
	itsFileText->SetBackColor(JGetCurrColormap()->GetWhiteColor());
}

/******************************************************************************
 Receive 

 ******************************************************************************/

void
GXChooseFileImportDialog::Receive
	(
	JBroadcaster* sender, 
	const Message& message
	)
{
	if (sender == itsFilterMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		itsFilterIndex = selection->GetIndex();
		}
		
	else if (sender == itsReloadButton && message.Is(JXButton::kPushed))
		{
		gPlotApp->ReloadImportModules();
		itsFilterMenu->RemoveAllItems();

		const JSize dirModCount = itsDir->GetInternalModuleCount();
		for (JSize i = 1; i <= dirModCount; i++)
			{
			itsFilterMenu->AppendItem(itsDir->GetInternalModuleName(i));
			}

		JPtrArray<JString>* names = gPlotApp->GetImportModules();
		const JSize strCount = names->GetElementCount();
		for (JSize i = 1; i <= strCount; i++)
			{
			itsFilterMenu->AppendItem(*(names->NthElement(i)));
			}
		itsFilterIndex = 1;
		itsFilterMenu->SetToPopupChoice(kTrue, itsFilterIndex);
		}
		
	else
		{
		JXDialogDirector::Receive(sender, message);
		}
}

/******************************************************************************
 GetFilterIndex 

 ******************************************************************************/

JIndex 
GXChooseFileImportDialog::GetFilterIndex()
{
	return itsFilterIndex;
}

/******************************************************************************
 GetFileText (public)

 ******************************************************************************/

const JString&
GXChooseFileImportDialog::GetFileText()
{
	return itsFileText->GetText();
}

/******************************************************************************
 ReadPrefs (protected)

 ******************************************************************************/

void
GXChooseFileImportDialog::ReadPrefs
	(
	istream& input
	)
{
	JIndex id;
	input >> id;
	if (id <= kFileModulePrefsVersionID)
		{
		input >> id;
		if (id <= itsFilterMenu->GetItemCount())
			{
			itsFilterIndex = id;
			itsFilterMenu->SetToPopupChoice(kTrue, itsFilterIndex);
			}
		JPoint loc;
		input >> loc;
		JCoordinate w;
		JCoordinate h;
		input >> w;
		input >> h;
		GetWindow()->SetSize(w, h);
		GetWindow()->Place(loc.x, loc.y);
		}
}

/******************************************************************************
 WritePrefs (protected)

 ******************************************************************************/

void
GXChooseFileImportDialog::WritePrefs
	(
	ostream& output
	)
	const
{
	output << kFileModulePrefsVersionID << ' ';
	output << itsFilterIndex << ' ';
	output << GetWindow()->GetDesktopLocation() << ' ';
	output << GetWindow()->GetFrameWidth() << ' ';
	output << GetWindow()->GetFrameHeight() << ' ';
}
