/******************************************************************************
 GXPlotApp.cc

	GXPlotApp class

	Copyright  1997 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "GXPlotApp.h"
#include "TestFloatTableDirector.h"
#include "GLGlobals.h"
#include "GLPrefsMgr.h"
#include "GLAboutDialog.h"

#include <JXDocumentManager.h>

#include <JUNIXDirEntry.h>
#include <JUNIXDirInfo.h>
#include <JPtrArray.h>
#include <JArray.h>
#include <JPoint.h>
#include <JUNIXPrefsFile.h>	
#include <JUserNotification.h>

#include <jXGlobals.h>

#include <jDirUtil.h>

#include <fstream.h>
#include <strstream.h>
#include <iostream.h>
#include <stdlib.h>
#include <jAssert.h>

GXPlotApp* gPlotApp = NULL;

const JCharacter* kCursorSubPath = "/cursormodule/";
const JCharacter* kDataSubPath = "/datamodule/";
const JCharacter* kImportSubPath = "/importmodule/";
const JCharacter* kExportSubPath = "/exportmodule/";
const JCharacter* kFitSubPath = "/fitmodule/";

static const JCharacter* kGloveFileSignature = "*** Glove File Format ***";

extern const JCharacter* kGloveVersionNumberStr;

/******************************************************************************
 Constructor

 ******************************************************************************/

GXPlotApp::GXPlotApp
	(
	int* 		argc, 
	char* 		argv[],
	JBoolean* 	displayAbout, 
	JString* 	prevVersStr
	)
	:
	JXApplication(argc, argv)
{
// Assumption - person has home dir, or no fileimpprogs
	gPlotApp = this;
	*displayAbout = InitGLGlobals(this);

	if (!*displayAbout)
		{
		*prevVersStr = (GetPrefsMgr())->GetGloveVersionStr();
		if (*prevVersStr == kGloveVersionNumberStr)
			{
			prevVersStr->Clear();
			}
		else
			{
			*displayAbout = kTrue;
			}
		}
	else
		{
		prevVersStr->Clear();
		}

//	itsFileImpProgs = new JPtrArray<JString>();

	JString homeDir;
	
	if (!JGetHomeDirectory(&homeDir))
		{
		homeDir == "/";		
		}
	
	JAppendDirSeparator(&homeDir);

	JXCreateDefaultDocumentManager(JXDocumentManager::kMetaShortcuts, kTrue);
	
/*
	JString fihome = homeDir + ".glove/fileimport/";
	foundFileImpDir = kFalse;
	if (JUNIXDirInfo::Create(fihome, &itsFileImpDir))
		{
		foundFileImpDir = kTrue;
		UpdateFileImpProgs();
		}
*/
	itsDirNumber = 1;
	
	itsModulePath = new JPtrArray<JString>;
	
	itsDataModules = new JPtrArray<JString>;
	itsCursorModules = new JPtrArray<JString>;
	itsImportModules = new JPtrArray<JString>;
	itsExportModules = new JPtrArray<JString>;
	itsFitModules = new JPtrArray<JString>;

	itsDataPathIndex = new JArray<JIndex>;
	itsCursorPathIndex = new JArray<JIndex>;
	itsImportPathIndex = new JArray<JIndex>;
	itsExportPathIndex = new JArray<JIndex>;
	itsFitPathIndex = new JArray<JIndex>;

	JString dmhome = homeDir + ".glove";
	JString* str = new JString(dmhome);
	itsModulePath->Append(str);
	str = new JString("/usr/local/lib/glove");
	itsModulePath->Append(str);
	
	ReloadDataModules();
	ReloadExportModules();
	ReloadCursorModules();
	ReloadFitModules();
	ReloadImportModules();
}

/******************************************************************************
 Destructor

 ******************************************************************************/

GXPlotApp::~GXPlotApp()
{
/*
	itsFileImpProgs->DeleteAll();
	delete itsFileImpProgs;
	
	if (foundFileImpDir)
		{
		delete itsFileImpDir;
		}
*/
	itsModulePath->DeleteAll();
	delete itsModulePath;

	itsDataModules->DeleteAll();
	delete itsDataModules;
	itsCursorModules->DeleteAll();
	delete itsCursorModules;
	itsImportModules->DeleteAll();
	delete itsImportModules;
	itsExportModules->DeleteAll();
	delete itsExportModules;
	itsFitModules->DeleteAll();
	delete itsFitModules;
	delete itsDataPathIndex;
	delete itsCursorPathIndex;
	delete itsImportPathIndex;
	delete itsExportPathIndex;
	delete itsFitPathIndex;
	DeleteGLGlobals();
}

/******************************************************************************
 ReloadImportModules

 ******************************************************************************/

void 
GXPlotApp::ReloadImportModules()
{
	itsImportModules->DeleteAll();
	itsImportPathIndex->RemoveAll();
	JUNIXDirInfo* info;
	
	const JSize count = itsModulePath->GetElementCount();
	for (JSize i = 1; i <= count; i++)
		{
		JString path = *(itsModulePath->NthElement(i)) + kImportSubPath;
		if (JUNIXDirInfo::Create(path, &info))
			{
			for (JSize j = 1; j <= info->GetEntryCount(); j++)
				{
				const JUNIXDirEntry& entry = info->GetEntry(j);
				if ( entry.IsExecutable() && !(entry.IsDirectory()) )
					{
					JString* str = new JString(entry.GetName());
					itsImportModules->Append(str);
					itsImportPathIndex->AppendElement(i);
					}
				}					
			delete info;
			}
		}
}

/******************************************************************************
 GetImportModulePath

 ******************************************************************************/

JBoolean
GXPlotApp::GetImportModulePath
	(
	const JIndex index,
	JString* path
	)
{
	assert (itsImportModules->IndexValid(index));
	
	JIndex mIndex = itsImportPathIndex->GetElement(index);
	*path = *(itsModulePath->NthElement(mIndex)) + kImportSubPath +
				*(itsImportModules->NthElement(index));
	return kTrue;
/*	
	if (foundFileImpDir)
		{
		assert( itsFileImpProgs->IndexValid(index) );
		
		JString str = itsFileImpDir->GetCWD();
		JAppendDirSeparator(&str);
		str += *(itsFileImpProgs->NthElement(index));
		*path = str;
		return kTrue;
		}
		
	return kFalse;
*/		
}

/******************************************************************************
 GetImportModules

 ******************************************************************************/

JPtrArray<JString>* 
GXPlotApp::GetImportModules()
{
//	if (foundFileImpDir)
//		{
//		itsFileImpDir->UpdateList();
//		}
	return itsImportModules;
}

/******************************************************************************
 Receive


void 
GXPlotApp::Receive
	(
	JBroadcaster* sender, 
	const Message& message
	)
{
	if (sender == itsFileImpDir && 
		(message.Is(JUNIXDirInfo::kContentsUpdated) ||
		message.Is(JUNIXDirInfo::kPermissionsChanged) ))
		{
		UpdateFileImpProgs();
		}
		
}
 ******************************************************************************/


/******************************************************************************
 DirectorClosed

 ******************************************************************************/

void 
GXPlotApp::DirectorClosed
	(
	JXDirector* theDirector
	)
{

}

/******************************************************************************
 UpdateFileImpProgs


void 
GXPlotApp::UpdateFileImpProgs()
{
	for (JSize i = 1; i <= itsFileImpDir->GetEntryCount(); i++)
		{
		const JUNIXDirEntry& entry = itsFileImpDir->GetEntry(i);
		if ( entry.IsExecutable() && !(entry.IsDirectory()) )
			{
			JString* str = new JString(entry.GetName());
			itsFileImpProgs->Append(str);
			}
		}
}
 ******************************************************************************/


/******************************************************************************
 NewFile

 ******************************************************************************/

void 
GXPlotApp::NewFile()
{
	JString str = "Untitled " + JString(itsDirNumber);
	itsDirNumber++;
	TestFloatTableDirector* tableDir = new TestFloatTableDirector(this, str);
	assert( tableDir != NULL);
	tableDir->Activate();
}

/******************************************************************************
 OpenFile

 ******************************************************************************/

void 
GXPlotApp::OpenFile
	(
	const JCharacter* filename
	)
{
	JXFileDocument* doc;
	if (JXGetDocumentManager()->FileDocumentIsOpen(filename, &doc))
		{
		doc->Activate();
		}
	else
		{
		ifstream is(filename);
		
		if (is.bad())
			{
			JGetUserNotification()->ReportError("Error opening file.");
			}
		else
			{
			JBoolean native = kTrue;	
			char line[255];
			is.getline(line,255);
			
			JString str(line);
			if (str != kGloveFileSignature)
				{
				native = kFalse;
				}
			TestFloatTableDirector* tableDir = new TestFloatTableDirector(this, filename, native);
			assert( tableDir != NULL);
			tableDir->LoadFile(filename);
			tableDir->Activate();
			}
		}
}

/******************************************************************************
 ReloadDataModules

 ******************************************************************************/

void 
GXPlotApp::ReloadDataModules()
{
	itsDataModules->DeleteAll();
	itsDataPathIndex->RemoveAll();
	JUNIXDirInfo* info;
	
	const JSize count = itsModulePath->GetElementCount();
	for (JSize i = 1; i <= count; i++)
		{
		JString path = *(itsModulePath->NthElement(i)) + kDataSubPath;
		if (JUNIXDirInfo::Create(path, &info))
			{
			for (JSize j = 1; j <= info->GetEntryCount(); j++)
				{
				const JUNIXDirEntry& entry = info->GetEntry(j);
				if ( entry.IsExecutable() && !(entry.IsDirectory()) )
					{
					JString* str = new JString(entry.GetName());
					itsDataModules->Append(str);
					itsDataPathIndex->AppendElement(i);
					}
				}					
			delete info;
			}
		}
}

/******************************************************************************
 GetDataModulePath

 ******************************************************************************/

JBoolean
GXPlotApp::GetDataModulePath
	(
	const JIndex index,
	JString* path
	)
{
	assert (itsDataModules->IndexValid(index));
	
	JIndex mIndex = itsDataPathIndex->GetElement(index);
	*path = *(itsModulePath->NthElement(mIndex)) + kDataSubPath + 
				*(itsDataModules->NthElement(index));
	return kTrue;
		
}

/******************************************************************************
 GetDataModules

 ******************************************************************************/

JPtrArray<JString>* 
GXPlotApp::GetDataModules()
{
	return itsDataModules;
}

/******************************************************************************
 ReloadCursorModules

 ******************************************************************************/

void 
GXPlotApp::ReloadCursorModules()
{
	itsCursorModules->DeleteAll();
	itsCursorPathIndex->RemoveAll();
	JUNIXDirInfo* info;
	
	const JSize count = itsModulePath->GetElementCount();
	for (JSize i = 1; i <= count; i++)
		{
		JString path = *(itsModulePath->NthElement(i)) + kCursorSubPath;
		if (JUNIXDirInfo::Create(path, &info))
			{
			for (JSize j = 1; j <= info->GetEntryCount(); j++)
				{
				const JUNIXDirEntry& entry = info->GetEntry(j);
				if ( entry.IsExecutable() && !(entry.IsDirectory()) )
					{
					JString* str = new JString(entry.GetName());
					itsCursorModules->Append(str);
					itsCursorPathIndex->AppendElement(i);
					}
				}					
			delete info;
			}
		}
}

/******************************************************************************
 GetCursorModulePath

 ******************************************************************************/

JBoolean
GXPlotApp::GetCursorModulePath
	(
	const JIndex index,
	JString* path
	)
{
	assert (itsCursorModules->IndexValid(index));
	
	JIndex mIndex = itsCursorPathIndex->GetElement(index);
	*path = *(itsModulePath->NthElement(mIndex)) + kCursorSubPath +
				*(itsCursorModules->NthElement(index));
	return kTrue;
		
}

/******************************************************************************
 GetCursorModules

 ******************************************************************************/

JPtrArray<JString>* 
GXPlotApp::GetCursorModules()
{
	return itsCursorModules;
}

/******************************************************************************
 ReloadExportModules

 ******************************************************************************/

void 
GXPlotApp::ReloadExportModules()
{
	itsExportModules->DeleteAll();
	itsExportPathIndex->RemoveAll();
	JUNIXDirInfo* info;
	
	const JSize count = itsModulePath->GetElementCount();
	for (JSize i = 1; i <= count; i++)
		{
		JString path = *(itsModulePath->NthElement(i)) + kExportSubPath;
		if (JUNIXDirInfo::Create(path, &info))
			{
			for (JSize j = 1; j <= info->GetEntryCount(); j++)
				{
				const JUNIXDirEntry& entry = info->GetEntry(j);
				if ( entry.IsExecutable() && !(entry.IsDirectory()) )
					{
					JString* str = new JString(entry.GetName());
					itsExportModules->Append(str);
					itsExportPathIndex->AppendElement(i);
					}
				}					
			delete info;
			}
		}
}

/******************************************************************************
 GetExportModulePath

 ******************************************************************************/

JBoolean
GXPlotApp::GetExportModulePath
	(
	const JIndex index,
	JString* path
	)
{
	assert (itsExportModules->IndexValid(index));
	
	JIndex mIndex = itsExportPathIndex->GetElement(index);
	*path = *(itsModulePath->NthElement(mIndex)) + kExportSubPath +
				*(itsExportModules->NthElement(index));
	return kTrue;
		
}

/******************************************************************************
 GetExportModules

 ******************************************************************************/

JPtrArray<JString>* 
GXPlotApp::GetExportModules()
{
	return itsExportModules;
}

/******************************************************************************
 ReloadFitModules

 ******************************************************************************/

void 
GXPlotApp::ReloadFitModules()
{
	itsFitModules->DeleteAll();
	itsFitPathIndex->RemoveAll();
	JUNIXDirInfo* info;
	
	const JSize count = itsModulePath->GetElementCount();
	for (JSize i = 1; i <= count; i++)
		{
		JString path = *(itsModulePath->NthElement(i)) + kFitSubPath;
		if (JUNIXDirInfo::Create(path, &info))
			{
			for (JSize j = 1; j <= info->GetEntryCount(); j++)
				{
				const JUNIXDirEntry& entry = info->GetEntry(j);
				if ( entry.IsExecutable() && !(entry.IsDirectory()) )
					{
					JString* str = new JString(entry.GetName());
					itsFitModules->Append(str);
					itsFitPathIndex->AppendElement(i);
					}
				}					
			delete info;
			}
		}
}

/******************************************************************************
 GetFitModulePath

 ******************************************************************************/

JBoolean
GXPlotApp::GetFitModulePath
	(
	const JIndex index,
	JString* path
	)
{
	assert (itsFitModules->IndexValid(index));
	
	JIndex mIndex = itsFitPathIndex->GetElement(index);
	*path = *(itsModulePath->NthElement(mIndex)) + kFitSubPath +
				*(itsFitModules->NthElement(index));
	return kTrue;
		
}

/******************************************************************************
 GetFitModules

 ******************************************************************************/

JPtrArray<JString>* 
GXPlotApp::GetFitModules()
{
	return itsFitModules;
}

/******************************************************************************
 DisplayAbout

	prevVersStr can be NULL.

 ******************************************************************************/

void
GXPlotApp::DisplayAbout
	(
	const JCharacter* prevVersStr
	)
{
	GLAboutDialog* dlog = new GLAboutDialog(this, prevVersStr);
	assert( dlog != NULL );
	dlog->BeginDialog();
}
