/******************************************************************************
 JXToolBar.cc

	BASE CLASS = public JXWidgetSet

	Copyright  1998 by Glenn W. Bach.  All rights reserved.
	
 *****************************************************************************/

#include <JXToolBar.h>
#include <JXToolbarButton.h>

#include <JXTextMenu.h>
#include <JXTextMenuData.h>
#include <JXImage.h>
#include <JXTimerTask.h>
#include <JXApplication.h>

#include <jXGlobals.h>

#include <jAssert.h>

const JCoordinate kButtConBuffer 	= 5;
const JCoordinate kButtConWidth 	= 30;

const JSize kTimerDelay				= 1000;

/******************************************************************************
 Constructor

 *****************************************************************************/

JXToolBar::JXToolBar
	(
	JXContainer* 		enclosure,
	const HSizingOption hSizing, 
	const VSizingOption vSizing,
	const JCoordinate 	x, 
	const JCoordinate 	y,
	const JCoordinate 	w, 
	const JCoordinate 	h
	)
	:
   JXWidgetSet(enclosure, hSizing, vSizing, x, y, w, h),
   itsNextButtonPosition(kButtConBuffer),
   itsInNewGroupMode(kTrue),
   itsTimerTask(NULL)
{
	assert(h >= 40);

	itsButtons = new JPtrArray<JXToolbarButton>;
	assert(itsButtons != NULL);

	itsMenus = new JPtrArray<JXMenu>;
	assert(itsMenus != NULL);
}

/******************************************************************************
 Destructor

 *****************************************************************************/

JXToolBar::~JXToolBar()
{
	delete itsTimerTask;
	delete itsButtons;
	delete itsMenus;
}

/******************************************************************************
 AppendButton

 ******************************************************************************/

void
JXToolBar::AppendButton
	(
	JXTextMenu* 	menu, 
	const JIndex 	index
	)
{
	JXToolbarButton* buttcon = 
		new JXToolbarButton(menu, index, this, 
					JXWidget::kFixedLeft, JXWidget::kFixedTop, 
					itsNextButtonPosition, kButtConBuffer, 
					kButtConWidth, kButtConWidth);
	assert(buttcon != NULL);
	ListenTo(buttcon);
	itsButtons->Append(buttcon);

	itsNextButtonPosition  += kButtConWidth;
	itsInNewGroupMode		= kFalse;
	
	const JXImage* image;
	if (menu->GetItemImage(index, &image))
		{
		buttcon->SetImage(const_cast<JXImage*>(image), kFalse);
		}

	buttcon->SetHint(menu->GetItemText(index));

	JIndex findex;
	if (!itsMenus->Find(menu, &findex))
		{
		itsMenus->Append(menu);
		}

	if (itsTimerTask == NULL)
		{
		itsTimerTask = new JXTimerTask(kTimerDelay);
		assert(itsTimerTask != NULL);
		JXGetApplication()->InstallIdleTask(itsTimerTask);
		ListenTo(itsTimerTask);
		}
}

/******************************************************************************
 NewGroup

 ******************************************************************************/

void
JXToolBar::NewGroup()
{
	if (itsInNewGroupMode)
		{
		return;
		}
	itsNextButtonPosition  += kButtConBuffer;
	itsInNewGroupMode		= kTrue;
}

/******************************************************************************
 Receive

 ******************************************************************************/

void 
JXToolBar::Receive
	(
	JBroadcaster* 	sender, 
	const Message&	message
	)
{
	if (message.Is(JXButton::kPushed))
		{
		const JSize count = itsButtons->GetElementCount();
		for (JSize i = 1; i <= count; i++)
			{
			JXToolbarButton* button = itsButtons->NthElement(i);
			if (sender == button)
				{
				button->GetMenu()->PrepareToOpenMenu();
				if (button->GetMenu()->itsBaseItemData->IsEnabled(button->GetMenuItem()))
					{
					button->GetMenu()->BroadcastSelection(button->GetMenuItem());
					}
				break;
				}
			}
		}
	else if (sender == itsTimerTask && message.Is(JXTimerTask::kTimerWentOff))
		{
		const JSize count = itsMenus->GetElementCount();
		for (JSize i = 1; i <= count; i++)
			{
			JXMenu* menu = itsMenus->NthElement(i);
			menu->PrepareToOpenMenu();
			}
		const JSize bcount = itsButtons->GetElementCount();
		for (JSize i = 1; i <= bcount; i++)
			{
			JXToolbarButton* button = itsButtons->NthElement(i);
			if (button->GetMenu()->itsBaseItemData->IsEnabled(button->GetMenuItem()))
				{
				button->Activate();
				}
			else
				{
				button->Deactivate();
				}
			}
		}
}