/******************************************************************************
 PlotDir.h

	Interface for the PlotDir class

	Copyright  1997 by Glenn Bach. 
	This code may be freely distributed, used, and modified without restriction.

 ******************************************************************************/

#ifndef _H_PlotDir
#define _H_PlotDir

#include <JXDocument.h>
#include <JXWindowDirector.h>
#include <JPtrArray.h>
#include <JArray.h>
#include <CurveStats.h>

class GlovePlotter;
class JString;
class ostream;
class istream;
class JFunction;
class GVarList;
class JXTextMenu;
class GXPlotFunctionDialog;
class JFitBase;
class FitParmsDir;
class GRaggedFloatTableData;
class JXFileDocument;
class JPlotDataBase;
class GloveHistoryDir;
class JXPSPrinter;
class FitModuleDialog;
class JPlotModuleFit;
class JPlotFitFunction;

class PlotDir : public JXDocument
{
public:

	PlotDir(JXDirector* supervisor, JXFileDocument* notifySupervisor, const JCharacter* filename, const JBoolean hideOnClose = kFalse);

	virtual ~PlotDir();
	
	GlovePlotter* 	GetPlot();
	void 				NewFileName(const JCharacter* filename);
	
	void 				WriteSetup(ostream& os);
	void 				ReadSetup(istream& is);
	
	void 				WriteData(ostream& os, GRaggedFloatTableData* data);
	void 				ReadData(istream& is, GRaggedFloatTableData* data);
	virtual JBoolean	NeedsSave() const;
	GloveHistoryDir*	GetSessionDir();
	JBoolean			AddFitModule(JPlotModuleFit* fit, JPlotDataBase* fitData);
	virtual void		SafetySave(const JXDocumentManager::SafetySaveReason reason);
	virtual JXPM		GetMenuIcon() const;

	virtual JBoolean	Close();

protected:

	virtual void		Receive(JBroadcaster* sender,
								const JBroadcaster::Message& message);
	virtual JBoolean	OKToClose();
	virtual void		DiscardChanges();

private:

	GlovePlotter* 				itsPlot;
	JString* 					itsFileName;
	GVarList*					itsVarList;
	JXTextMenu*					itsPlotMenu;
	JXTextMenu*					itsFitParmsMenu;
	JXTextMenu*					itsCurveMenu;
	JXTextMenu*					itsCurveExpMenu;
	JXTextMenu*					itsCurveQuadMenu;
	JXTextMenu*					itsCurveModuleMenu;
	JXTextMenu*					itsDiffMenu;
	JXTextMenu*					itsHelpMenu;
	JIndex						itsXVarIndex;
	GXPlotFunctionDialog*		itsFunctionDialog;
	JPtrArray<JFitBase>*		itsFits;
	FitParmsDir*				itsFitParmsDir;
	JArray<CurveStats>*			itsCurveStats;
	JArray<JIndex>*				itsCurveIndeces;
	GCurveType					itsCurrentCurveType;
	JXFileDocument*				itsSupervisor;
	JPtrArray<PlotDir>*			itsDiffDirs;
	JBoolean					itsHideOnClose;
	GloveHistoryDir*			itsSessionDir;
	JXPSPrinter*				itsPrinter;
	JBoolean					itsIsPrintAll;
	FitModuleDialog*			itsFitModuleDialog;
	JIndex						itsCurveForFit;

	JBoolean					itsPlotIsClosing;

private:

	void 				ReadCurves(istream& is, GRaggedFloatTableData* data);

	void 				PlotFunction(JFunction* f);
	void				HandlePlotMenu(const JIndex index);
	void				CreateFunction();
	void				SelectFitModule();
	void				UpdateFitParmsMenu();

	void				UpdateDiffMenu();

	void				UpdateCurveMenu();
	void				HandleCurveMenu(const JIndex index);
	void				HandleCurveExpMenu(const JIndex index);
	void				HandleCurveQuadMenu(const JIndex index);
	void				HandleCurveModuleMenu(const JIndex index);

	void				HandleHelpMenu(const JIndex index);

	void				HandleCurveRemoved(const JIndex index);
	void				HandleCurveAdded();
	void				RemoveFit(const JIndex index);
	void				NewFit(const JIndex plotindex, const GCurveFitType type);
	void				AddDiffCurve(JPlotDataBase* ddata);
	void				AddFit(JPlotFitFunction* fit, const JIndex plotIndex, 
								const GCurveFitType type);
	

	// not allowed

	PlotDir(const PlotDir& source);
	const PlotDir& operator=(const PlotDir& source);
};

/******************************************************************************
 GetPlot

 ******************************************************************************/

inline GlovePlotter*
PlotDir::GetPlot()
{
	return itsPlot;
}	

/******************************************************************************
 GetSessionDir

 ******************************************************************************/

inline GloveHistoryDir*
PlotDir::GetSessionDir()
{
	return itsSessionDir;
}
	
#endif
