/******************************************************************************
 TestFloatTableDirector.h

	Interface for the TestFloatTableDirector class

	Written by John Lindal.

 ******************************************************************************/

#ifndef _H_TestFloatTableDirector
#define _H_TestFloatTableDirector

#include <JXFileDocument.h>
#include <JString.h>
#include <JPtrArray.h>

class GRaggedFloatTableData;
class GXRaggedFloatTable;
class JXTextMenu;
class JXPSPrinter;
class JXDialogDirector;
class GXChooseFileImportDialog;
class PlotDir;
class ifstream;
class JXScrollbarSet;
class GXGetDelimiterDialog;

class TestFloatTableDirector : public JXFileDocument
{
public:

	TestFloatTableDirector(JXDirector* supervisor, const JCharacter* fileName,
							const JBoolean onDisk = kFalse);

	virtual ~TestFloatTableDirector();

	void CreateNewPlot(	const JIndex type,
						JArray<JFloat>* xCol, JArray<JFloat>* xErrCol,
						JArray<JFloat>* yCol, JArray<JFloat>* yErrCol,
						JBoolean linked, const JCharacter* label );
						
	void AddToPlot( const JIndex plotIndex, const JIndex type,
					JArray<JFloat>* xCol, JArray<JFloat>* xErrCol,
					JArray<JFloat>* yCol, JArray<JFloat>* yErrCol,
					JBoolean linked, const JCharacter* label );
	
	void 					GetPlotNames(JPtrArray<JString>& names);
	void					LoadFile(const JCharacter* filename);
	GRaggedFloatTableData*	GetData();
	const JCharacter*		GetInternalModuleName(const JIndex index);
	JSize					GetInternalModuleCount();

protected:

	virtual void	Receive(JBroadcaster* sender, const Message& message);
	virtual void	DirectorClosed(JXDirector* theDirector);
	virtual void	WriteTextFile(ostream& output, const JBoolean safetySave) const;
	virtual void	DiscardChanges();
			
private:

	JXPSPrinter*				itsPrinter;
	GRaggedFloatTableData*		itsData;
	GXRaggedFloatTable*			itsTable;			// owned by its enclosure
	JXTextMenu*					itsFileMenu;		// owned by the menu bar
	JXTextMenu*					itsExportMenu;
	JXTextMenu*					itsHelpMenu;
	
	GXChooseFileImportDialog*	itsFileImportDialog;
	GXGetDelimiterDialog*		itsDelimiterDialog;
	JString						itsCurrentFileName;

	JPtrArray<PlotDir>* itsPlotWindows;
	JSize				itsPlotNumber;
	JBoolean			itsListenToData;

	JXScrollbarSet*		itsScrollbarSet;

// begin JXLayout


// end JXLayout

private:

	void	BuildWindow();

	void	UpdateFileMenu();
	void	HandleFileMenu(const JIndex item);

	void	UpdateExportMenu();
	void	HandleExportMenu(const JIndex item);

	void	HandleHelpMenu(const JIndex item);

	void	LoadNativeFile(ifstream& is);
	void	LoadImportFile();
	void	LoadDelimitedFile();
	void	LoadInternalFile(const JIndex index);
	
	void	ChooseFileFilter();
	void	ReadPlotData(istream& is);

	// not allowed

	TestFloatTableDirector(const TestFloatTableDirector& source);
	const TestFloatTableDirector& operator=(const TestFloatTableDirector& source);
};

/******************************************************************************
 GetData

 ******************************************************************************/

inline GRaggedFloatTableData*	
TestFloatTableDirector::GetData()
{
	return itsData;
}

#endif
